/******************************************************************************
*******************************************************************************

	Space Impakto DS
	relminator
	http://rel.betterwebber.com


	Cbullet class


*******************************************************************************
******************************************************************************/


#include <nds.h>

#include "Cobject.h"
#include "Cbullet.h"

/******************************************************************************


******************************************************************************/
void state_normal(Cbullet *b)
{

	b->dx += b->gx; 
	b->dy += b->gy;					
	b->x += b->dx;  
	b->y += b->dy;	 
	
	if (b->delay >= 0)
	{
		if (b->counter == b->delay)		//stop
		{
			b->dx_old = b->dx;
			b->dy_old = b->dy;
			b->dx = 0;
			b->dy = 0;     	
		}

		if ( (b->counter > b->delay) && (b->counter < b->delay  * 3) )  // wait	  
		{
			b->dx = 0;
			b->dy = 0;     	
		}
		
		if (b->counter == b->delay * 3)		//move again			
		{
			if (b->smart)
			{
				int _x = b->x >> 12;
				int _y = b->y >> 12;
				int brad_angle = atan2Lookup( b->target->x - _x, b->target->y - _y );
				
				b->dx = ( cosLerp(brad_angle) * b->speed ) >> 12;
	            b->dy = ( sinLerp(brad_angle) * b->speed ) >> 12;
			}
			else
			{
				b->dx = b->dx_old;
				b->dy = b->dy_old;
			}
		}
	}
	
}

/******************************************************************************


******************************************************************************/
void state_accel(Cbullet *b)
{

	b->x_accel += b->gx;
	b->y_accel += b->gy;

	b->dx += b->x_accel;
	b->dy += b->y_accel;

	b->dx = b->dx - ( (b->dx * FRICTION) >> 12 );   
	b->dy = b->dy - ( (b->dy * FRICTION) >> 12 );

	b->x += b->dx; 
	b->y += b->dy;	

	if (b->delay >= 0)
	{
		if (b->counter == b->delay)		//stop
		{
			b->dx_old = b->dx;
			b->dy_old = b->dy;
			b->dx = 0;
			b->dy = 0;     	
		}

		if ( (b->counter > b->delay) && (b->counter < b->delay  * 3) )  // wait	  
		{
			b->dx = 0;
			b->dy = 0;     	
		}

		if (b->counter == b->delay * 3)		//move again			
		{
			if (b->smart)
			{
				int _x = b->x >> 12;
				int _y = b->y >> 12;
				int brad_angle = atan2Lookup( b->target->x - _x, b->target->y - _y );
				b->dx = 0;
				b->dy = 0;
				b->x_accel = ( cosLerp(brad_angle) * b->speed ) >> 14;   // * 0.2
	            b->y_accel = ( sinLerp(brad_angle) * b->speed ) >> 14;	
			}
			else
			{
				b->dx = b->dx_old;
				b->dy = b->dy_old;
				b->x_accel = b->x_accel >> 1;
				b->y_accel = b->y_accel >> 1;
			}
		}
	}
	
}

/******************************************************************************


******************************************************************************/
void state_invaccel(Cbullet *b)
{

	b->dx += b->gx;
	b->dy += b->gy;
	
	if ( ((b->dx * b->dx + b->dy * b->dy) >> 12) > 4096 )  			 
	{
		b->dx = (b->dx * INVFRICTION) >> 12;
		b->dy = (b->dy * INVFRICTION) >> 12;
	}
	
	b->x += b->dx;  
	b->y += b->dy;
	
	if (b->delay >= 0)
	{
		if (b->counter == b->delay)		//stop
		{
			b->dx_old = b->dx;
			b->dy_old = b->dy;
			b->dx = 0;
			b->dy = 0;     	
		}

		if ( (b->counter > b->delay) && (b->counter < b->delay  * 3) )  // wait	  
		{
			b->dx = 0;
			b->dy = 0;     	
		}
		
		if (b->counter == b->delay * 3)		//move again			
		{
			if (b->smart)
			{
				int _x = b->x >> 12;
				int _y = b->y >> 12;
				int brad_angle = atan2Lookup( b->target->x - _x, b->target->y - _y );
				
				b->dx = ( cosLerp(brad_angle) * b->speed ) >> 12;
	            b->dy = ( sinLerp(brad_angle) * b->speed ) >> 12;
			}
			else
			{
				b->dx = b->dx_old;
				b->dy = b->dy_old;
			}
		}
	}
	
}

/******************************************************************************


******************************************************************************/
Cbullet::Cbullet()
{

	x = 0;
	y = 0;
	dx = 0;
	dy = 0;
	gx = 0;
	gy = 0;
	active = false;
	counter = 0;
	delay = -1;
	rotation = 0;
	energy = 0;
	death_ID = 0;
	state = EB_NORMAL;
	smart = false;
	do_state = state_normal;

}

/******************************************************************************

_x and _y are not FP

******************************************************************************/
Cbullet::Cbullet(int _x, int _y)
{

	x = _x << 12;
	y = _y << 12;
	dx = 0;
	dy = 0;
	gx = 0;
	gy = 0;
	active = false;
	counter = 0;
	delay = -1;
	rotation = 0;
	energy = 0;
	death_ID = 0;
	state = EB_NORMAL;
	smart = false;
	do_state = state_normal;
	
}


/******************************************************************************

_targetx and _targety are not FP
_bspeed is .12 FP

******************************************************************************/
void Cbullet::update()
{

	
	
	counter++;
	do_state(this);
	
	if ( (x < B_MIN_X) || (x > B_MAX_X) || (y < B_MIN_Y) || (y > B_MAX_Y)) 
	{
		active = false;
		return;
	}
	// animate 
	if ( anim_counter >= anim_delay)
	{
		frame++;
		if ( frame >= num_frames)
			frame = 0;
		
		anim_counter = -1;
	}

	// inc counter
	anim_counter++;
	
	resolve_aabb_f32();
	box.shrink(2600);

	
}

/******************************************************************************


******************************************************************************/
void Cbullet::set_state(E_BULLET_TYPE _state)
{

	state = _state;
	switch (state)
	{
		case EB_NORMAL:
			do_state = state_normal;
			break;
		case EB_ACCEL:
			do_state = state_accel;
			break;
		case EB_INVACCEL:
			do_state = state_invaccel;
			break;
		default:
			do_state = state_normal;
	}

}


